<?php
// https://roadcubepublicapi.docs.apiary.io/#reference/users/initialize/initialize-user's-registration

function roadcube_get_countries()
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/countries?phone_code=00880",
    array(
      'method' => 'GET',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
      ),
    )
  );
  if (is_wp_error($response)) {
    $error_code = array_key_first($response->errors);
    $error_message = $response->errors[$error_code][0];
    echo esc_attr($error_message);
  } else {
    $countries = json_decode($response['body'], true);
    update_option('roadcube_country_data', $countries['data']);
  }
}
function roadcube_user_register_init(array $user_data)
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $url = isset($user_data['mobile_verify']) ? "https://api.roadcube.io/v1/p/users/registration/mobile-verification" : "https://api.roadcube.io/v1/p/users/registration/init";
  unset($user_data['mobile_verify']);
  $response = wp_remote_get(
    $url,
    array(
      'method' => 'POST',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
        'Content-Type' => 'application/json'
      ),
      'body' => wp_json_encode($user_data),
    )
  );
  if (is_wp_error($response)) {
    $error_code = array_key_first($response->errors);
    $error_message = $response->errors[$error_code][0];
    echo esc_attr($error_message);
  } else {
    $data = json_decode($response['body'], true);

    if ($data['status'] == "success") {
      $user_data['user_reg_id'] = $data['data']['user']['user_registration_identifier'];
      $user_data['status'] = "success";
      $user_data['data'] = $data;
      return $user_data;
    } else {
      $data['stage'] = "initial_registration";
      return $data;
    }
  }
}
function roadcube_user_register(array $user_data)
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/users/registration/finalize",
    array(
      'method' => 'POST',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
        'Content-Type' => 'application/json'
      ),
      'body' => wp_json_encode($user_data),
    )
  );
  if (is_wp_error($response)) {
    $error_code = array_key_first($response->errors);
    $error_message = $response->errors[$error_code][0];
    echo esc_attr($error_message);
  } else {
    $data = json_decode($response['body'], true);
    if (isset($data['status'])) {
      if ($data['status'] == "success") {
        return $user_data;
      } else {
        $data['stage'] = "final_registration";
        return $data;
      }
    }
  }
}
function roadcube_get_the_point($user_mobile)
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $dataset = [
    "user" => $user_mobile
  ];
  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/users/points",
    array(
      'method' => 'POST',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
        'Content-Type' => 'application/json'
      ),
      'body' => wp_json_encode($dataset),
    )
  );
  if (is_wp_error($response)) {
    $error_code = array_key_first($response->errors);
    $error_message = $response->errors[$error_code][0];
    echo esc_attr($error_message);
  } else {
    $data = json_decode($response['body'], true);
    return $data;
  }
}
function roadcube_get_available_coupons($page = 1)
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/coupons?page={$page}",
    array(
      'method' => 'GET',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
      ),
    )
  );
  if (is_wp_error($response)) {
    $error_code = array_key_first($response->errors);
    $error_message = $response->errors[$error_code][0];
    echo esc_attr($error_message);
  } else {
    $data = json_decode($response['body'], true);
    return $data;
  }
}
function roadcube_coupon_claim($user_mobile, $coupon_id)
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $dataset = [
    "user" => $user_mobile,
    "coupon_id" => $coupon_id
  ];
  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/users/coupon-claims",
    array(
      'method' => 'POST',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
        'Content-Type' => 'application/json'
      ),
      'body' => wp_json_encode($dataset),
    )
  );
  if (is_wp_error($response)) {
    $error_code = array_key_first($response->errors);
    $error_message = $response->errors[$error_code][0];
    echo esc_attr($error_message);
  } else {
    $data = json_decode($response['body'], true);
    $data['dataset'] = $dataset;
    return $data;
  }
}
function roadcube_user_coupon_claim($user_mobile)
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $dataset = [
    "user" => $user_mobile,
    // "coupon_id" => $coupon_id
  ];
  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/users/user-coupon-claims?page=1",
    array(
      'method' => 'POST',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
        'Content-Type' => 'application/json'
      ),
      'body' => wp_json_encode($dataset),
    )
  );
  if (is_wp_error($response)) {
    $error_code = array_key_first($response->errors);
    $error_message = $response->errors[$error_code][0];
    echo esc_attr($error_message);
  } else {
    $data = json_decode($response['body'], true);
    $data['dataset'] = $dataset;
    return $data;
  }
}


function roadcube_checkout_new_transaction_call($user, $amount, $order_id, $phone = '')
{
  // Sync the user if not already synced

  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $store_id = Coupon_Claimer::roadcube_get_setting('store_id');
  $is_mob_attached = false;

  if (!roadcube_check_user_synced($user)) {
    roadcube_create_user_by_email($user);
  }

  if ($phone) {
    $dataset_ph = [
      "email" => $user,
      "mobile" => $phone
    ];

    $response = wp_remote_get(
      "https://api.roadcube.io/v1/p/users/attach-mobile/without-verification",
      array(
        'method' => 'POST',
        'headers' => array(
          'X-API-TOKEN' => $api_key,
          'Content-Type' => 'application/json'
        ),
        'body' => wp_json_encode($dataset_ph),
      )
    );
    if (!empty($response)) {
      $is_mob_attached = true;
    }
  }

  $products = array();
  $order = wc_get_order($order_id);
  foreach ($order->get_items() as $item_id => $item) {
    $product = $item->get_product();
    $product_type = $product->get_type();
    if ($product->get_type() == 'variation') {
      $parent_product = wc_get_product($product->get_parent_id());
      $variation_price = $product->get_price();
      $products[] = array(
        'product_id' => get_post_meta($parent_product->get_id(), 'roadcube_product_id', true), // Parent product ID
        'retail_price' => $variation_price,
        'quantity' => $item->get_quantity()
      );
    } elseif ($product->get_type() == 'variable') {
      $variations = $product->get_available_variations();
      foreach ($variations as $variation) {
        $products[] = array(
          'product_id' => get_post_meta($variation['variation_id'], 'roadcube_product_id', true),
          'retail_price' => $variation['display_price'],
          'quantity' => $item->get_quantity()
        );
      }
    } else {
      $products[] = array(
        'product_id' => get_post_meta($product->get_id(), 'roadcube_product_id', true),
        'retail_price' => $product->get_price(),
        'quantity' => $item->get_quantity()
      );
    }
  }
  $dataset = [
    "user" => ($is_mob_attached) ? $phone : $user
  ];
  $trans_type = Coupon_Claimer::roadcube_get_setting('roadcube_create_transaction_settings') ?: 'amount';
  if ($trans_type == 'amount') {
    $dataset['amount'] = $amount;
  } else {
    $dataset['products'] = $products;
  }

  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/stores/{$store_id}/transactions/new",
    array(
      'method' => 'POST',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
        'Content-Type' => 'application/json'
      ),
      'body' => wp_json_encode($dataset),
    )
  );
  $response['request'] = $dataset;
  update_option('roadcube_trans_create_log', $response);
  return $response;
}
function roadcube_checkout_cancel_transaction_call($trans_id)
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $store_id = Coupon_Claimer::roadcube_get_setting('store_id');
  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/stores/{$store_id}/transactions/cancel",
    array(
      'method' => 'POST',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
        'Content-Type' => 'application/json'
      ),
      'body' => wp_json_encode(
        array(
          'transaction_id' => $trans_id
        )
      )
    )
  );
  return json_decode($response['body'], true);
}

function roadcube_get_user_available_coupons($page = 1, $user_email = false)
{
  if (!$user_email) {
    return false;
  }
  $dataset = array(
    'user' => $user_email
  );
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/users/coupons-to-claim/?page='.$page",
    array(
      'method' => 'POST',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
        'Content-Type' => 'application/json'
      ),
      'body' => wp_json_encode($dataset),
    )
  );
  if (is_wp_error($response)) {
    $error_code = array_key_first($response->errors);
    $error_message = $response->errors[$error_code][0];
    return ['status' => 'error', 'message' => "cURL Error #:" . $error_message];
  } else {
    $data = json_decode($response['body'], true);
    return $data;
  }
}

// Register user by email
function roadcube_create_user_by_email($email)
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  if (!$email) return;
  $password = (string) wp_rand(100000, 999999);
  $dataset = array(
    'email'     => $email,
    'gender'    => 'male',
    'password'  => $password,
    'password_confirmation' => $password,
    'birthday'  => '1999-12-30'
  );
  $response = wp_remote_get(
    "https://api.roadcube.io/v1/p/users/registration/email",
    array(
      'method' => 'POST',
      'headers' => array(
        'X-API-TOKEN' => $api_key,
        'Content-Type' => 'application/json'
      ),
      'body' => wp_json_encode($dataset),
    )
  );

  if (is_wp_error($response)) {
    $error_code = array_key_first($response->errors);
    $error_message = $response->errors[$error_code][0];
    return ['status' => 'error', 'message' => "cURL Error #:" . $error_message];
  } else {
    $data = json_decode($response['body'], true);
    return $data;
  }
}

function roadcube_check_user_synced($email)
{
  $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
  $response = wp_remote_get("https://api.roadcube.io/v1/p/users/check?email={$email}", [
    'headers' => [
      'X-API-TOKEN' => $api_key,
    ],
  ]);

  if (is_wp_error($response)) {
    return $response->get_error_message();
  }

  $body = wp_remote_retrieve_body($response);
  $data = json_decode($body, true);

  return !empty($data['exists']) && $data['exists'] === true;
}
