<?php
function roadcube_sync_all_prev_products(){
    $query = new WP_Query( array(
        'post_type' => 'product',
        'post_status'  => 'publish',
        'posts_per_page' => -1
    ) );

    $product_ids = [];
    while($query->have_posts()){
        $query->the_post();
        $product_id = get_the_ID();
        $product_ids[] = $product_id;
    }
    $i = 1;

    foreach(array_chunk($product_ids,10) as $product_ids_chunk) {
        wp_schedule_single_event(time() + 100 * $i, 'roadcube_synce_products', array( $product_ids_chunk ) );

        $i++;
    }
}
add_action('roadcube_synce_products','roadcube_synce_products_callback');
function roadcube_synce_products_callback( $product_ids ){

    $product_log = get_option('roadcube_product_log',[]);
    $product_log = [];
    foreach($product_ids as $product_id){
        $added_log = roadcube_save_product_callback($product_id,true);
        $product_log[] = $added_log;
        update_option('roadcube_product_log',$product_log);
    }
}
function roadcube_get_product_category(){
    $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
    $store_id = Coupon_Claimer::roadcube_get_setting('store_id');

    $response = wp_remote_get("https://api.roadcube.io/v1/p/stores/{$store_id}/products?page=1",
      array(
        'method' => 'GET',
        'headers' => array(
          'X-API-TOKEN' => $api_key,
        ),
      )
    );

    $output = json_decode($response['body'],true);
    update_option('roadcube_category_log',$output);
    if( isset($output['status']) && $output['status'] == 'success' ){
        $category_id = false;
        foreach( $output['data']['products'] as $product ) {
            if( isset($product['product_category']) && is_array($product['product_category']) ) {
                $category_id = $product['product_category']['product_category_id'];
                break;
            }
        }
        return $category_id;
    } else {
        return false;
    }
}
function roadcube_create_product( array $dataset ) {
    $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
    $store_id = Coupon_Claimer::roadcube_get_setting('store_id');
    $product_id = '';
    $method = 'POST';
    if( isset($dataset['product_id']) ){
        $product_id = $dataset['product_id'];
        $method = 'PUT';
    }

    $response = wp_remote_get("https://api.roadcube.io/v1/p/stores/{$store_id}/products/{$product_id}",
      array(
        'method' => $method,
        'headers' => array(
          'X-API-TOKEN' => $api_key,
          'Content-Type' => 'application/json'
        ),
        'body' => wp_json_encode($dataset)
      )
    );

    // curl_close($curl);

    return json_decode($response['body'],true);
}

add_action('save_post_product','roadcube_save_product_callback');
function roadcube_save_product_callback( $product_id, $sync = false ){
    // Fetching the category id - Start

    $api_key = Coupon_Claimer::roadcube_get_setting('api_key');
    $store_id = Coupon_Claimer::roadcube_get_setting('store_id');

    $response = wp_remote_get("https://api.roadcube.io/v1/p/stores/{$store_id}/product-categories/",
      array(
        'headers' => array(
          'X-API-TOKEN' => $api_key,
          'Content-Type' => 'application/json'
        ),
      )
    );


    $response_data = json_decode($response['body'], true);
    $product_category_id = ($response_data['data']['product_categories'][0]['product_category_id']) ? $response_data['data']['product_categories'][0]['product_category_id'] : 2907;
    // Fetching the category id - End
    $product = wc_get_product( $product_id );

    if ( $product->is_type( 'simple' ) ) {
          // Simple product logic
          $sale_price = $sync ? $product->get_sale_price() : ( isset( $_POST['_sale_price'] ) ? sanitize_text_field( wp_unslash( $_POST['_sale_price'] ) ) : '' );
          $regular_price = $sync ? $product->get_regular_price() : ( isset( $_POST['_regular_price'] ) ? sanitize_text_field( wp_unslash( $_POST['_regular_price'] ) ) : '' );

    } elseif ( $product->is_type( 'variable' ) ) {
          // Variable product logic
          $variations = $product->get_available_variations();

          foreach ( $variations as $variation ) {
              $variation_product = wc_get_product( $variation['variation_id'] );
              $sale_price = $variation_product->get_sale_price();
              $regular_price = $variation_product->get_regular_price();
          }
    }

    $current_currency = get_woocommerce_currency();
    if($current_currency == "EUR"){
        $regular_price = str_replace('.', '', $regular_price);
        $regular_price = str_replace(',', '.', $regular_price);

        $sale_price = str_replace('.', '', $sale_price);
        $sale_price = str_replace(',', '.', $sale_price);
    }else{
        $regular_price = str_replace(',', '', $regular_price);
        $sale_price = str_replace(',', '', $sale_price);
    }

    $title = $sync ? $product->get_title() : ( isset( $_POST['post_title'] ) ? sanitize_text_field( wp_unslash( $_POST['post_title'] ) ) : '' );

    // $category_id = get_option('roadcube_store_cat_id') ?: roadcube_get_product_category();
    $category_id = $product_category_id;
    update_option('roadcube_store_cat_id',$category_id);
    $product_data = array(
        'published' => true,
        'name' => array(
            'en' => $title,
            'it' => $title,
            'el' => $title
        ),
        'description' => array(
            'en' => $title,
            'el' => $title,
            'it' => $title
        ),
        'retail_price' => $regular_price ?: $sale_price,
        'wholesale_price' => $sale_price ?: $regular_price,
        'product_category_id' => $category_id,
        'group_product' => false
    );
    if( get_post_meta($product_id, 'roadcube_product_id', true) ) {
        $product_data['product_id'] = get_post_meta($product_id, 'roadcube_product_id', true);
    }

    $created_product = roadcube_create_product($product_data);

    $created_product_id = $created_product['data']['product']['product_id'];
    update_post_meta( $product_id, 'roadcube_product_id', $created_product_id );
    update_post_meta( $product_id, 'roadcube_product_created_data', $created_product );
    $created_product['body'] = $product_data;
    $created_product['cat_resp'] = $category_id;
    return $created_product;
}